/*
 * xrick/src/syssnd.c
 *
 * Copyright (C) 1998-2002 BigOrno (bigorno@bigorno.net). All rights reserved.
 *
 * The use and distribution terms for this software are contained in the file
 * named README, which can be found in the root of this distribution. By
 * using this software in any fashion, you are agreeing to be bound by the
 * terms of this license.
 *
 * You must not remove this notice, or any other, from this software.
 */
#include "./s2d/s2d.h"
using namespace s2d;

#include "../inc/GameEngine.h"

extern GameEngine* pEng;

//#include <stdlib.h>
//#include <stdio.h>

#include "config.h"

#ifdef ENABLE_SOUND

#define _strcmp s2d::strcmp

#include "system.h"
#include "game.h"
#include "syssnd.h"
#include "debug.h"
#include "data.h"

/*
#include "../../data/ent7_raw.h"
#include "../../data/ent8_raw.h"
#include "../../data/ent4_raw.h"
#include "../../data/ent5_raw.h"
#include "../../data/ent6_raw.h"
#include "../../data/die_raw.h"
#include "../../data/ent0_raw.h"
#include "../../data/ent1_raw.h"
#include "../../data/ent2_raw.h"
#include "../../data/ent3_raw.h"
#include "../../data/bonus_raw.h"
#include "../../data/box_raw.h"
#include "../../data/crawl_raw.h"
#include "../../data/explode_raw.h"
#include "../../data/jump_raw.h"
#include "../../data/pad_raw.h"
#include "../../data/sbonus1_raw.h"
#include "../../data/stick_raw.h"
#include "../../data/walk_raw.h"
#include "../../data/bombshht_raw.h"
#include "../../data/bullet_raw.h"
#include "../../data/gameover_raw.h"
#include "../../data/sbonus2_raw.h"
#include "../../data/tune5_raw.h"
#include "../../data/tune4_raw.h"
#include "../../data/tune3_raw.h"
#include "../../data/tune2_raw.h"
#include "../../data/tune1_raw.h"
#include "../../data/tune6_raw.h"
#include "../../data/tune7_raw.h"
#include "../../data/tune0_raw.h"
*/

static U8 isAudioActive = FALSE;
static U8 sndMute = FALSE;  /* mute flag */

U8 noSnd;

void
syssnd_init(void)
{
  noSnd = 1;
}

/*
 * Shutdown
 */
void
syssnd_shutdown(void)
{
  if (!isAudioActive) return;

  isAudioActive = FALSE;
}

/*
 * Toggle mute
 *
 * When muted, sounds are still managed but not sent to the dsp, hence
 * it is possible to un-mute at any time.
 */
void
syssnd_toggleMute(void)
{
  sndMute = !sndMute;
}

/*
 * Play a sound
 *
 * loop: number of times the sound should be played, -1 to loop forever
 * returns: channel number, or -1 if none was available
 *
 * NOTE if sound is already playing, simply reset it (i.e. can not have
 * twice the same sound playing -- tends to become noisy when too many
 * bad guys die at the same time).
 */
S8 c=0;

S8
syssnd_play(sound_t *sound, S8 loop)
{
//  if(sound->buf != NULL) 
  {
    if (!_strcmp(sound->name,"tune0.raw") || !_strcmp(sound->name,"tune1.raw") || !_strcmp(sound->name,"tune2.raw") 
     || !_strcmp(sound->name,"tune3.raw") || !_strcmp(sound->name,"tune4.raw") || !_strcmp(sound->name,"tune5.raw") 
     || !_strcmp(sound->name,"tune6.raw") || !_strcmp(sound->name,"tune7.raw")) { 
      c=0; 
      pEng->PlayMusic(0,false);
    } 
    else {
      pEng->PlaySound(sound->channel);
    }

	  c = (c+1) & 0x7;
    if (c==0) c=1;
  }
  //sound->channel = c;

  return c;
}

/*
 * Pause
 *
 * pause: TRUE or FALSE
 * clear: TRUE to cleanup all sounds and make sure we start from scratch
 */
void
syssnd_pause(U8 pause, U8 clear)
{
  if (!isAudioActive) return;

}

/*
 * Stop a sound
 */
void
syssnd_stopsound(sound_t *sound)
{
	if (!sound) return;

	//m_pAudioPlayer->Stop(sound->channel);
}

/*
 * Stops all channels.
 */
void
syssnd_stopall(void)
{
	//m_pAudioPlayer->StopAll();
}

/*
 * Load a sound.
 */
sound_t *
syssnd_load(char *name)
{
	sound_t *s;

	/* alloc sound */
	s = (sound_t*)malloc(sizeof(sound_t));

//	s->name = (char*) malloc(_strlen(name) + 1);
  s2d::strcpy(s->name, name);
  s->len = 0; s->buf = NULL;

  if (!_strcmp(s->name,"tune0.raw") || !_strcmp(s->name,"tune1.raw") || !_strcmp(s->name,"tune2.raw") 
   || !_strcmp(s->name,"tune3.raw") || !_strcmp(s->name,"tune4.raw") || !_strcmp(s->name,"tune5.raw") 
   || !_strcmp(s->name,"tune6.raw") || !_strcmp(s->name,"tune7.raw")) { 
    pEng->LoadMusic( 0, name );
    s->channel = 0;
  } 
  else {
    pEng->LoadMusic( noSnd, name );
    s->channel = noSnd;
    noSnd++;
  }

	return s;
}

/*
 *
 */
void
syssnd_free(sound_t *s)
{
	if (!s) return;
  //free(s->name);
  free(s);
}

#endif /* ENABLE_SOUND */

/* eof */

